from __future__ import annotations

'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''

# the idea is that I have a gui where I can set the level and service to debug
# a print will then accept two additional optional parameter, level and service
# if the current debugls has BOTH level and service flagged, the print will be printed
# note that it is possible to omit the level or service, meaning, it does not matter
# NOTE: this should be able to save the settings...



from app_config import AppConfig
from glob_gui import TkToplevel, SolarCheckbox  
import tkinter as tk
import tkinter.ttk as ttk


lvl_trace='trace'
lvl_debug='debug'
lvl_notice='notice'
lvl_rxdata='rxdata'
lvl_txdata='txdata'

lvl_list = {
    lvl_trace: 'Trace',
    lvl_debug : 'Debug',
    lvl_notice: 'Notice',
    lvl_rxdata: 'Rx Data',
    lvl_txdata: 'Tx Data',
    }

srv_login="login"
srv_heartbeat="heartbeat"

srv_list = { 
        srv_login      : 'Login',
        srv_heartbeat  : "Heart beat"
        }



# ---------------------------------------------------------------------------
# This is the main instance, it will need to pick up the saved config values and save config at the end
# It is a floating Frame, seems easier...
class DbglsGui():
        
    # ----------------------------------------------------------
    # constructor
    def __init__(self, stat : 'Qpystat' ):
        self.stat=stat
        
        self.stat.appconfig.addMyselfToSavables(self)

        self._toplevel = TkToplevel("Debug Configuration")
        
        frame = ttk.Frame(self._toplevel)

        frame.columnconfigure(0, weight=1)
        frame.rowconfigure(2, weight=1)
    
        self.levels = DbglsPanel(frame,lvl_list,'Levels')
        self.levels.grid(row=0, column=0, sticky='NSEW')
        
        self.services = DbglsPanel(frame,srv_list,'Services')
        self.services.grid(row=1, column=0, sticky='NSEW')
        
        self.applog = LogPanel(self.stat, frame, "Application Log")
        self.applog.grid(row=2, column=0, sticky='NSEW')
        
        frame.pack(fill='both',expand=True)
        
    # --------------------------------------------------------------------
    # show a window that has been hidden using the withdraw method
    def deiconify(self):
        self._toplevel.deiconify()    
        

    # --------------------------------------------------------------------
    # hide completely a window
    def withdraw(self):
        self._toplevel.withdraw()    
    
    # -------------------------------------------------------------------
    # prints something on the application log
    def println(self, msg):
        self.applog.println(msg)    

    # implement the config savable
    def appImportConfig(self, cfg : AppConfig ):
        adict = cfg.getDictValue("dbgls_cfg", {})
        
        try:
            self.services.importConfig(adict['dbgls_svc'])
            self.levels.importConfig(adict['dbgls_level'])    
            self._toplevel.setGeometry(adict['gui_cfg'])
        except Exception as _exc :
            pass            

    # implement the config savable
    def appSaveConfig(self, cfg : AppConfig ):
        adict = dict()

        adict['gui_cfg'] = self._toplevel.getGeometry()
        adict['dbgls_svc'] = self.services.getConfig()
        adict['dbgls_level'] = self.levels.getConfig()

        cfg.setDictValue("dbgls_cfg", adict )            
            

    # ------------------------------------------------------------------------
    # Will return true if I should log at the given service and level
    def mustLog (self, service, level):
        logsvc = self.services.mustLog(service)
        loglvl = self.levels.mustLog(level)
        
        return logsvc and loglvl

# -------------------------------------------------------------------------
# The panel holds a bunch of checkboxes and on top there is a label
# , borderwidth=3, relief='ridge'
#    cbox.pack(side='left')

class DbglsPanel(ttk.Frame):

    # -------------------------------------------------------------
    # Constructor
    def __init__(self, container, check_list, heading ):
        ttk.Frame.__init__(self, container, padding="3p.3p", borderwidth=3, relief='ridge')     
        
        label = ttk.Label(self, text=heading, anchor='w')
        label.config(font =("Courier", 14))
        label.pack(fill='x', side='top')

        # this will hold all check boxes
        frame = ttk.Frame(self)
        frame.pack(side='top',fill='both', expand=True)
        
        self.checklist = list()

        wrow=0
        wcol=0
        
        # This way I get the name and label in the for loop syntax
        # See https://realpython.com/iterate-through-dictionary-python/
        for name,label in check_list.items():
            cbox = DbglsCheckbox(frame, name, label)
            cbox.grid(row=wrow, column=wcol,sticky ="nw")

            wcol += 1
            
            if wcol >= 6:
                wrow += 1
                wcol = 0

            self.checklist.append(cbox)

    # -------------------------------------------------------------------
    # return true if name is null or the associated checkbox is True
    def mustLog (self, name ):
        if name:
            for cbox in self.checklist:
                if cbox.name == name:
                    return cbox.isChecked()
        else:
            return True

    # --------------------------------------------------------------------
    # returns a dictionary with name and status of the checkbox
    def getConfig(self):
        dic = {}
        
        for abox in self.checklist:
            dic[abox.name] = abox.isChecked()
        
        return dic;
    
    # ---------------------------------------------------------------------
    # given a dictionary with name : value, reload the status
    def importConfig(self, panel_config : dict ):
        for name,status in panel_config.items():
            self.setStatus(name, status)
        
    # ----------------------------------------------------------------------
    # Set the status of a single checkbox    
    def setStatus (self, name, status ):
        for cbox in self.checklist:
            if cbox.name == name:
                cbox.setChecked(status)
                break

# ----------------------------------------------------------------------
# Need to wrap the checkbox into something usable...
# it has to have a name and a simple boolean value..    
# NOTE that the layout is done at caller level, NOT here
class DbglsCheckbox(ttk.Checkbutton):    
    def __init__(self, container, cname, text ):
        
        # need this to then match it 
        self.name=cname
        
        # this value can be obtained outside the swing thread
        self.cvalBoolean = False
        
        # this is needed since this is the logic used to propagate state in tkinter...
        self.cval = tk.BooleanVar(name=cname)
        
        # and this defines that a callback will be called when the value changes
        self.cval.trace_add("write", self.cboxCallback)
        
        ttk.Checkbutton.__init__(self, container, text=text, onvalue=True, offvalue=False, variable=self.cval)     
        
    def cboxCallback(self, *args):
        self.cvalBoolean = self.cval.get()
            
    def isChecked(self) -> bool:
        return self.cvalBoolean    
        
    def setChecked(self, abol):
        self.cval.set(abol)
        
# --------------------------------------------------------------------------------------------------
# The idea here is to have a log window that has a scrollable part, up to a certain number of rows
# stat is the general holder of application vars, container is where to put this log
# NOTE: you MUST apply the proper layout of THIS class on the caller side
# Since python is shy on making things private, I just make this class a Frame
# Not only, but since you have to layout on parent you eventually end up on forwarding quite a few useless methods
# so, keep this to extend ttk.Frame, this is the only reasonable thing to do with python
# Note that since the printing is done in the swing thread, there is no need to hold a lock
class LogPanel(ttk.Frame):
    
    # ------------------------------------------------------------------------
    # You can decide the max number of lines to show
    # to pad contente you can use padding="3p.3p" or you can set padx=3, pady=3
    def __init__(self, stat : 'Qpystat', container, name, maxlines=5000):
        ttk.Frame.__init__(self, master=container,padding=3)     

        self.stat=stat
        self.maxlines=maxlines

        self.checkPaused : SolarCheckbox = None;
        
        # Create label inside the frame
        self._newNorthPanel(self, name).pack(fill='x',pady=2)

        # create a text inside the frame
        self._newTextPanel(self).pack(fill='both',expand=True) 
        
    # -----------------------------------------------------------
    # Mostly a reminder that this method is available everywhere there is a 'frame'   
    def runOnGuiIdle(self, func, *args ):
        self.after_idle(func, *args)
        
    # ------------------------------------------------------------
    # create a new north panel
    # The layout of this panel is done at PARENT level
    def _newNorthPanel(self,parent, name):
        apanel = ttk.Frame(parent)

        # Create label inside the apanel
        ttk.Label(apanel, text=name, anchor='w', font=("Courier", 14)).pack(side='left')

        # add a checkbox to pause the writing
        self.checkPaused=SolarCheckbox(apanel,'Pause')
        self.checkPaused.pack(side='left')
    
        ttk.Button(apanel,text="Clear" , command=self._clickClear ).pack(side='left')
    
        return apanel

    # --------------------------------------------------------------
    # When button is click, clear, it is thread safe since is in gui thread
    def _clickClear(self):
        self.text.delete('1.0', 'end')
        
    # ------------------------------------------------------------
    # this creates a new panel to hold both the text and scrollbar    
    def _newTextPanel(self,parent):
        # The layout of this apanel is done at caller level
        apanel = ttk.Frame(parent)
        
        # Create text widget and specify size.
        self.text = tk.Text(apanel, height = 10, width = 80)
        self.text.pack(fill='both',expand=True,side='left')
        
        scroll_bar = ttk.Scrollbar(apanel,orient='vertical',command=self.text.yview)
        scroll_bar.pack(fill='y', expand=False, side='left')
        
        self.text['yscrollcommand'] = scroll_bar.set
        
        self.setWrap(False)
                
        return apanel
        
        
    # --------------------------------------------------------------    
    def getTextString(self) -> str :
        astring = self.text.get("1.0",'end-1c')    
        return astring
        
    # -------------------------------------------------------------
    # true, enables wrap, false nowrap
    def setWrap (self, enabled):
        if enabled:
            self.text.config(wrap='word')    
        else:
            self.text.config(wrap='none')
        
    # --------------------------------------------------------------------
    # Use this one to actually print something
    # @param addnl if true a newline will be added after the given text
    def _println_gui (self, msg : str, addnl : bool ):

        if self.checkPaused.isChecked():
            return

        if not self.text.winfo_exists():
            # this may happen if in closing mode and messages want to be logged
            # if the widget is destroyed then print into the standard out
            # NOTE this will NOT happen if you call the window.destroy() of the main GUI from a gui idle callback
            print('logui wnot: ',msg)
            return 
        
        endpos=self.text.index('end')
        split=endpos.split('.')
        lines = int(split[0])
        
        if lines > self.maxlines :
            delindex=lines - self.maxlines 
            self.text.delete('1.0', "%d.end" % delindex)
            
        self.text.insert(tk.END, msg)
        
        if addnl:
            self.text.insert(tk.END, "\n")
        
        self.text.see(tk.END)

    # -------------------------------------------------------------------------
    # Always run the print on GUI idle, this solves a lot of issues on threading
    def println(self,msg):
        if len(msg) > 0:
            self.runOnGuiIdle(self._println_gui,msg,True)

    # -------------------------------------------------------------------------
    # this does NOT add a newline at the end
    def write(self,msg):
        if len(msg) > 0:
            self.runOnGuiIdle(self._println_gui,msg, False)

        