from __future__ import annotations

'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''

from datetime import datetime, timezone
from os import path
from pathlib import Path
import socket
import time

from PIL import Image 

import tkinter as tk
 

# ----------------------------------------------------------
# Returns the size part of a self.geometry() string
# 1089x402+-1206+380 should result into 1089, 402, -1206, 380
def splitGeometrySize(geom : str ):
    pieces = geom.split('+')
    
    xy = pieces[0]
#    posx = int(pieces[1])
#    posy= int(pieces[2])

    pieces = xy.split('x')
    
    wx = int(pieces[0])
    wy = int(pieces[1])

    return wx,wy


# -------------------------------------------------------------------
# This should allow to pick up data files in both the scripted and bundled version
# @param filename is a simple file name, no path that will be resolved in the directory of THIS module
def getThisFullpath( filename ):
    return path.abspath(path.join(path.dirname(__file__), filename))





# -------------------------------------------------------------------
# return a PhotoImage that pick up the file from where this module is
# You have to put the png files where this module is
# If for some reason the image is not found then an empty image is returned
def getPhotoImage(filename):
    try:
        fpath=getThisFullpath(filename)
        return tk.PhotoImage(file = fpath)
    except Exception as _exc:
        print("getPhotoImage: cannot find ",filename)
        return tk.PhotoImage(file="") 
        

# -------------------------------------------------------------------
# Someone needs an Image ... 
def getImage(filename) -> Image: 
    try:
        fpath=getThisFullpath(filename)
        return Image.open(fpath)
    except Exception as _exc:
        print("getImage: cannot find ",filename)
        return Image() 


# ------------------------------------------------------------------
# Should return the full path, relative to user home directory
def getHomeFullPath(filename):
    home = str(Path.home())
    return path.abspath(path.join(home, filename))





# ----------------------------------------------------------------------
# the idea is that this file holds the shared part of the system
# especially relevant is the qpystat that hold the state seen by the client and the server
def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(0)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except Exception:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP


# ---------------------------------------------------------------------
# To make it type safe and clear
# return the current time as seconds from the epoch
def getCurrentTime_s() -> int :
    afloat = time.time()
    return int(afloat)

# ----------------------------------------------------
# Attempt to retrieve the given key from the dictionary
# return the default value if not found, no exception
def getDictStringValue(adict : dict, key : str , defvalue : str ) -> str:
    try:
        return str(adict[key])
    except Exception as _exc:
        return defvalue
    
# ----------------------------------------------------
# get a value as an int
def getDictIntValue(adict : dict, key , defvalue : int ) -> int:
    try:
        return int(adict[key])
    except Exception as _exc:
        return defvalue



# --------------------------------------------------------------------
# get the current time a number of seconds since the epoch
# This is mostly here to remember the way python does it...
# NOTE: The time returned is the time NOW that is in UTC, tested equivalent to
# dt = datetime.now(timezone.utc)
# dt.timestamp()
def getUtcTimestamp () -> int:
    afloat = time.time()
    return int(afloat)

# ---------------------------------------------------------------------
# This returns a datetime
# It is basically, whatever is display in the computer, as it is
# it does not have timezone information and therefore... it is a mess
# https://docs.python.org/3/library/datetime.html
def getDatetimeNow() -> datetime:
    return datetime.now()

# ---------------------------------------------------------------------
# This returns a datetime that is rooted in UTC
# https://docs.python.org/3/library/datetime.html
def getUtcDatetimeNow() -> datetime:
    return datetime.now(timezone.utc)

# ---------------------------------------------------------------------
# Convert from unix timestamp (number of seconds since epoch) to a ISO string
# The incoming timestamp is supposed to be in UTC timezone, this is normally the case if coming from a sqllite3
def fromUtcTimestampToIsoString(time_s : int) -> str:
    dt = datetime.fromtimestamp(time_s,timezone.utc)
    return dt.strftime('%Y-%m-%d %H:%M:%S')

# -----------------------------------------------------------------------
# Convert from a UTC string to the unix time (seconds) assuming we are in UTC timezone
# PYTHON time handling is a mess in all languages.... especially python
def fromUtcIsoStringToTimestamp ( datetime_str : str ) -> int:
    dt = datetime.fromisoformat(datetime_str)
    dt = dt.replace(tzinfo=timezone.utc)
    af = dt.timestamp()
    return int(af)

# ---------------------------------------------------------------------
# Convert from unix timestamp (number of seconds since epoch) to a ISO string
def fromTimestampToIsoString(time_s : int) -> str:
    dt = datetime.fromtimestamp(time_s)
    return dt.strftime('%Y-%m-%d %H:%M:%S')

# -----------------------------------------------------------------------
# Convert from a string to the unix time (seconds) 
# PYTHON time handling is a mess in all languages.... especially python
def fromIsoStringToTimestamp ( datetime_str : str ) -> int:
    dt = datetime.fromisoformat(datetime_str)
    af = dt.timestamp()
    return int(af)


    
# ----------------------------------------------------------------------
# I have a unix timestamp and I basically want to strip the hours, minutes, seconds
# This sequence of operation is correct, do not touch the timezone issue at all and it works
def fromTimestampToDateTimestamp(datetime_s : int) -> int:    
    dt = datetime.fromtimestamp(datetime_s)
    dt.replace(hour=0, minute=0, second=0)
    af = dt.timestamp()
    return int(af)


# -----------------------------------------------------------------
# @return 0 -> 15 if valid, 16 if invalid
def fromHexDigitToByte ( digit ):
    if digit == '0':
        return 0;
    elif digit == '1':
        return 1;
    elif digit == '2':
        return 2;
    elif digit == '3':
        return 3;
    elif digit == '4':
        return 4;
    elif digit == '5':
        return 5;
    elif digit == '6':
        return 6;
    elif digit == '7':
        return 7;
    elif digit == '8':
        return 8;
    elif digit == '9':
        return 9;
    elif digit == 'A' or digit == 'a':
        return 10;
    elif digit == 'B' or digit == 'b':
        return 11;
    elif digit == 'C' or digit == 'c':
        return 12;
    elif digit == 'D' or digit == 'd':
        return 13;
    elif digit == 'E' or digit == 'e':
        return 14;
    elif digit == 'F' or digit == 'f':
        return 15;
    else:
        return 16;


# -----------------------------------------------------------------
# @return 0 -> 15 if valid, 16 if invalid
def fromDtmfDigitToByte ( digit ):
    if digit == '0':
        return 0;
    elif digit == '1':
        return 1;
    elif digit == '2':
        return 2;
    elif digit == '3':
        return 3;
    elif digit == '4':
        return 4;
    elif digit == '5':
        return 5;
    elif digit == '6':
        return 6;
    elif digit == '7':
        return 7;
    elif digit == '8':
        return 8;
    elif digit == '9':
        return 9;
    elif digit == 'A' or digit == 'a':
        return 10;
    elif digit == 'B' or digit == 'b':
        return 11;
    elif digit == 'C' or digit == 'c':
        return 12;
    elif digit == 'D' or digit == 'd':
        return 13;
    elif digit == '*':
        return 14;
    elif digit == '#':
        return 15;
    else:
        return 16;




"""
# -------------------------------------------------------------
# Since the image must end up into the ..... maybe not 
class ButtonImage(ttk.Button()):
    def __init__(self, parent, image_png, *options ):
        
        animage = tk.PhotoImage(file = image_png)
        # AHHHHH dammed snakes, it MUST be an instance variable since otherwise it get out of scope and it disappear!!!
        # subsample returns one pixel every the number given of pixels, so, 2 actually half the image
        self.sampled = animage.subsample(2, 2)
                    
        ttk.Button.__init__(self, parent, image=self.sampled, *options)

"""

