'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.

To add tk support in linux

apt-get install python3-tk


Hold classes and functions used to manage the gui
'''
# --------------------------------------------------------------------
# This would be a private class, but python is lacking object isolation
# Note that it is for all purposes a Tk, meaning that this is slightly redundant but may be useful in the future
# @param app_title: a string to display on the top window
# @param icon_name: the name of a icon file to use for systray and/or app icon
# @param on_closing_fun function to call when the user tries to close
# @param enable_systray if true then enable systray 

from __future__ import annotations

import os
from tkinter import ttk, BooleanVar
from typing import List

import glob_fun
from ipp_parser import EEblock
import tkinter as tk


class GuiRoot(tk.Tk):
    def __init__(self, stat : 'Qpystat', app_title, icon_name, on_closing_fun, enable_systray = False ):  
        tk.Tk.__init__(self)

        self.title(app_title)
        
        self.stat=stat
        
        self.app_title = app_title
        self.icon_name = icon_name
        self.on_closing_fun = on_closing_fun

        # systray can be enabled only if icon name is given        
        self.enable_systray=icon_name and enable_systray
        
        self.w_resizable=True

        # I need a list of windows to hide/show when requested        
        self.hide_show_list : List(TkToplevel) = []
        self.hide_show_list.append(self)

        self.setRootIcon()
        self.enableSystray()
    
    # --------------------------------------------------------------------------------------------------------
    # just to make it clear what the purpose is, terminate the gui loop
    def gui_destroy(self):
        self.destroy()
    

    # ----------------------------------------------------------------------
    # attempt enable systray if it is what is requested
    # if no systray docking is requested add the required on_closing fun
    # the idea is that when the window is closing it is equivalent of writing quit
    # this is VERY important since it allows to save windows position before win destroy
    # If nothing is defined I define a self way to close the gui, so, it is in the right thread
    def enableSystray(self):
        if self.on_closing_fun:
            self.protocol("WM_DELETE_WINDOW", self.on_closing_fun)
        else:
            self.protocol("WM_DELETE_WINDOW", self.gui_destroy)
        
    # ---------------------------------------------------------------------
    # When on systray mode and a request to show is posted
    def systray_show_window(self, icon, item):
        
        # the icon must be stop and created again
        icon.stop()
        
        # this has to run on swing thread
        self.after_idle(self.after_idle_deiconify)
    
    # --------------------------------------------------------------------
    # callback on all windows to deiconify
    def after_idle_deiconify(self):
        
        for win in self.hide_show_list: 
            win.deiconify()
    
    # -----------------------------------------------------------------
    # When in systray mode and quit is selected we arrive at this point
    def systray_quit_window(self, icon, item):
        
        # the icon must be stop and created again
        icon.stop()
        
        if self.on_closing_fun:
            self.on_closing_fun()
        else:
            self.destroy()
    
    
    # ----------------------------------------------------------------------
    # This will set a nice icon to the root window and attempt to setup systray if requested
    # note that it is not possible to enable systray if there is no icon
    # To have windows to use this icon for the application you need to add this
    
    # myappid = 'pbxsemplice.release.v0' # arbitrary string
    # ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

    # apparently, there are quite some compatibility issues
    # the problem with this is that it needs PIL, another library to load...
    # imgicon = PhotoImage(file=self.icon_name)
    # self.call('wm', 'iconphoto', self, imgicon)  

    
    # somewhere in main
    def setRootIcon(self):
        
        if not self.icon_name:
            return 
        
        try:
            if os.name == 'nt':
                icon_fname = self.icon_name+'.ico'
                self.wm_iconbitmap(bitmap=icon_fname, default=icon_fname)
            else:
                icon_fname = '@'+self.icon_name+'.xbm'
                self.wm_iconbitmap(bitmap=icon_fname)
                
        except Exception as _exc:
            print("setRootIcon: cannot find ",icon_fname)
    
    # ----------------------------------------------------------------------
    # apparently, there is no way to know if a window is resizable or not 
    def setResizable(self, i_resizable : bool ):
        self.w_resizable=bool(i_resizable)
        # PYTHON how to call superclass methods
        super().resizable(self.w_resizable,self.w_resizable)

    def isResizable(self) -> bool:
        return self.w_resizable
        
    # ---------------------------------------------------------------------
    # Mostly here to remember how to run a function at later time in swing thread
    # NOTE that you name the function to call and then pass the arguments
    # eg: stat.runGuiIdle(stat.extension.setExtensionStatus,extnum,status)    
    def runIdle(self, func, *args ):
        self.after_idle(func, *args)
        
    # ---------------------------------------------------------------------
    # this is instead when you want to apply a specific delay    
    def runLater(self, ms, func, *args ):
        self.after(ms, func, *args)

    # ---------------------------------------------------------
    # attempt to set the geometry of the window, if the new geometry is bigger then current
    # the parameter is a dictionary where the geometry key should be
    # an example of what comes in is  "geometry": "1089x402+-1206+380" 
    # Also: The current geometry is available ONLY after GUI is UP, so, you have to adjust with a timer, after application is up and running
    # Also: Since this is the Root window it must NOT be possible to hide it on creation, you will not be able to get it back 
    def setGeometry(self, propd ):
        try:
            # pick up current geometry
            mx,my = glob_fun.splitGeometrySize(self.geometry())

            geom = propd['window_geometry']
            wx,wy = glob_fun.splitGeometrySize(geom)
            
            if self.isResizable() and wx >= mx and wy >= my:
                self.geometry(geom)
                
#            wst = propd['window_state']
#            if wst:
#                self.state(wst)
                
        except Exception as exc:
            print("GuiRoot.setGeometry: ",exc)
            
    # ----------------------------------------------------
    # return the equivalent geometry info as a dictionary
    # If given the info will be appended to the given dictionary
    def getGeometry(self, have_dic=None):

        if have_dic:
            geodic=have_dic
        else:
            geodic=dict()
        
        geodic['window_geometry'] = self.geometry()
        geodic['window_state'] = self.state()
        
        return geodic
            


 
# ==================================================================================
# the point is that we always need a title and no window close, otherwise the object disappear
# So, a close will result into a window withdraw that can be reversed later, if needed
class TkToplevel(tk.Toplevel):

    # -----------------------------------------------------------
    def __init__(self, title ):
        tk.Toplevel.__init__(self)
        
        # we need some sort of title
        self.title(title)

        # this is the way to disable the close button
        self.protocol("WM_DELETE_WINDOW", self.wm_delete_window_fun)

    # -----------------------------------------------------------
    # Called when someone click on the "close window"
    # You can get the object back to display using deiconify()
    def wm_delete_window_fun(self):
        self.withdraw()    

    
    # ---------------------------------------------------------
    # attempt to set the geometry of the window, if the new geomerty is bigger then current
    # the parameter is a dictionary where the geometry key should be
    # an example of what comes in is  "geometry": "1089x402+-1206+380" 
    # Also, note that the current geometry is available ONLY after gui is UP, so, you have to 
    # adjust with a timer, after application is up and running 
    def setGeometry(self, propd ):
        try:
            #mx,my = glob_fun.splitGeometrySize(self.geometry())

            geom = propd['window_geometry']
            #wx,wy = glob_fun.splitGeometrySize(geom)
            
            #if wx >= mx and wy >= my:
            self.geometry(geom)
                
            if not self.winfo_viewable():
                # do not set window state if it is not viewable
                return 
            
            st = propd['window_state']
            if st:
                self.state(st)
                
        except Exception as exc:
            print("tktoplevel.setGeometry: ",exc)
            
    # ----------------------------------------------------
    # return the equivalent geometry info as a dictionary
    # If given the info will be appended to the given dictionary
    def getGeometry(self, have_dic=None):

        if have_dic:
            geodic=have_dic
        else:
            geodic=dict()
                    
        geodic['window_geometry'] = self.geometry()
        geodic['window_state'] = self.state()

        return geodic
            


# -------------------------------------------------------
# Quite often there is the need to have a confirmation popup
# It just shows the text and an OK button that will close the window        
# use
# import tkinter.messagebox as mbox 
# mbox.showinfo("Attenzione", "La applicazione si chiude dalla schermata principale")
#
"""
class TkPopupMessage(tk.Toplevel):
    def __init__(self, parent, text ):
        tk.Toplevel.__init__(self,parent)
        
        # we need some sort of title
        self.title("Messaggio")
        
        workPanel = ttk.Frame(self)
        
        label=tk.Label(workPanel, text=text, font=('Mistral 18 bold'))
        label.grid(row=0, column=0,padx=(20,20),pady=(30,30))

        okb=ttk.Button(workPanel,text="OK",command=self.ok_click)
        okb.grid(row=1, column=0,padx=(20,20),pady=(0,30))

        workPanel.pack()
        
    def ok_click(self):
        self.destroy()    
        
"""        
        


# --------------------------------------------------------------------
# This class should behave similarly to a Swing BorderLayout
# the way it works is to create one of this passing the proper parent
# then declare how it will be "placed" in the parent with something like this
# panel.pack(fill=tk.BOTH,expand=True)
# NOTE that if you do not fill and expand then it will not resize when needed...
# then you can "add" the pieces you wish, using this logic
# ttk.Label(panel.pnorth, text="NORTH",borderwidth=1,relief="ridge").pack()
# ttk.Label(panel.pleft, text="LEFT",borderwidth=1,relief="ridge").pack()
# ttk.Label(panel.pcenter, text="CENTER",borderwidth=1,relief="ridge").pack()
# ttk.Label(panel.pright, text="RIGHT",borderwidth=1,relief="ridge").pack()
# ttk.Label(panel.psouth, text="SOUTH",borderwidth=1,relief="ridge").pack()
# Clearly, you have to tell to subparts HOW to pack, you may not wish to expand...
        
class BorderLayout(ttk.Frame):
    def __init__(self, container ):
        tk.Frame.__init__(self, container)
        
        self.panel_list = []
        
        self.pnorth = ttk.Frame(self)
        self.pnorth.pack(fill='x',expand=False)
        self.panel_list.append(self.pnorth)

        # this should be a private variable, python is child crap
        self._fmiddle = ttk.Frame(self)
        self._fmiddle.pack(fill='both',expand=True)

        self.pleft = ttk.Frame(self._fmiddle)
        self.pleft.pack(fill='y',expand=False,side='right')
        self.panel_list.append(self.pleft)
        
        self.pcenter = ttk.Frame(self._fmiddle)
        self.pcenter.pack(fill='both',expand=True,side='right')
        self.panel_list.append(self.pcenter)
        
        self.pright = ttk.Frame(self._fmiddle)
        self.pright.pack(fill='y',expand=False,side='right')
        self.panel_list.append(self.pright)
        
        self.psouth = ttk.Frame(self)
        self.psouth.pack(fill='x',expand=False)
        self.panel_list.append(self.psouth)
        
    # --------------------------------------------------
    # Use as debugging method, show the border of the inner frames, so you can 
    # guess how the layout is happening    
    def showBorder(self):
        for panel in self.panel_list:
            panel['borderwidth'] = 1
            panel['relief'] = 'ridge'
        
                    
                    
                    
# -----------------------------------------------------------
# Wrap the image handling all in this class
# NOTE: this is a tk.Button since I need to easily hide border                    
class ImageButton(tk.Button):
    def __init__(self, container, image_fname, **params):

        # AHHHHH dammed snakes, it MUST be an instance variable since otherwise it get out of scope and it disappear!!!
        # subsample returns one pixel every the number given of pixels, so, 2 actually half the image
        self.btn_ico = glob_fun.getPhotoImage(image_fname).subsample(2, 2)
        tk.Button.__init__(self, container, image=self.btn_ico, relief='flat', **params )    




# -------------------------------------------------------------------
# Column definition is stored into an object since I need to switch the sorting for a given column
# This is used in TreeView to display a table
class TableColdef:

    # ----------------------------------------------------------------
    # Constructor
    def __init__(self, qcol_name, col_label, width, stretch=False ):
        self.col_name=qcol_name
        self.col_label=col_label
        self.sort_desc=True
        self.width=width
        self.stretch=stretch
        
    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.col_name+":"+self.col_label+":"+str(self.sort_desc)

# =================================================================================
# Need to wrap the checkbox into something usable...
# it has to have a name and a simple boolean value..    
# NOTE that the layout is done at caller level, NOT here
class SolarCheckbox(ttk.Checkbutton):    
    
    # -------------------------------------------------------------------------------
    # @parent the parent panel where to bind this checkbox
    # @param cname the name to be used to identify the checkbox, can be None if not used
    # @param text the text to display to the user
    def __init__(self, parent, text, cname=None ):

        # I need the text later
        self.text=text

        # need this to then match it, it can be an id, may be needed
        self.name=cname
        
        # this value can be obtained outside the swing thread
        self.cvalBoolean = False
        
        # this is needed since this is the logic used to propagate state in tkinter...
        # NOTE that I do NOT use the name since it may conflict if it is the same for different check
        self.cval = BooleanVar()
        
        # and this defines that a callback will be called when the value changes
        self.cval.trace_add("write", self._cboxCallback)
        
        # create the checkbox and bind it to the above cval
        ttk.Checkbutton.__init__(self, parent, text=text, onvalue=True, offvalue=False, variable=self.cval)     
        
    def _cboxCallback(self, *args):
        self.cvalBoolean = self.cval.get()
            
    def isChecked(self) -> bool:
        return self.cvalBoolean    
        
    # --------------------------------------------------------------------
    # if you pass an empty or illegal value nothing will change
    def setChecked(self, abol ):
        try:
            self.cval.set(abol)
        except:
            pass


class TV_Entry(ttk.Entry):
    # -------------------------------------------------------------------------------
    # @parent the parent panel where to bind this fentry
    # @param the text variable to bind to
    def __init__(self, parent, textvar, **kwargs ):
        
        afont=ttk.Style().lookup("TEntry", "font")

        if afont:        
            ttk.Entry.__init__(self, parent, textvariable=textvar, font=afont, **kwargs)
        else:
            ttk.Entry.__init__(self, parent, textvariable=textvar, **kwargs)
        
        
# ===============================================================================
# A frame that has grid layout as defauult layout, with specific methods     
# since this crap language cannot handle post increment
# idiots, reinventing the wheel as an hexagon   
class GridFrame(ttk.Frame):
    
    # ---------------------------------------------------------------------------
    def __init__(self, parent, **params):
        ttk.Frame.__init__(self, parent, **params)
        
        self._current_row=0;
        self._current_column=0;
        self._current_sticky='nsew'
        self.padx = 2
        
    # ---------------------------------------------------------------------------
    # Call this when you wish to go to the next row
    def nextRow(self):
        self._current_row += 1
        self._current_column = 0
        
    # ---------------------------------------------------------------------------
    # If you ever need to go to the next column, without adding a component
    def nextColumn(self):
        self._current_column += 1
                        
    # ---------------------------------------------------------------------------
    def addItem(self, jcomponent, **gridparams):
        jcomponent.grid(row=self._current_row, column = self._current_column, sticky=self._current_sticky)
        
        if self.padx:
            jcomponent.grid(padx=self.padx)
            
        self._current_column += 1
        
        
        
# ========================================================================
# A generic ancestor, it is usually useful to have a comman framework 
# This is the GUI for a block !
# It is a Frame that can be put somewhere and can be raised
# subclasses MUST add the gui to this 

class EEblockGui (ttk.Frame):

    # ----------------------------------------------------------------
    def __init__(self, parent_frame : ttk.Frame, ee_start_a : int, ee_len : int ):
        ttk.Frame.__init__(self, parent_frame,padding='3',borderwidth=1, relief='solid')
        
        self.ee_block : EEblock = EEblock(ee_start_a, ee_len, bytearray())
        
        #ttk.Label(self,text='0x{:02X} '.format(self.ee_block.ee_start_a)  ).pack(side='left')
        

    # --------------------------------------------------------------------
    # attempt to update the block, if the block is really for me
    # NOTE that a write eill return a block that is empty
    # @return true if it managed to do so
    
    def _updateBlock(self, i_block : EEblock ) ->bool:
        
        if not i_block:
            return False
        
        if i_block.get_data_len() < 1:
            return False
        
        if ( i_block.ee_start_a != self.ee_block.ee_start_a ):
            return False
        
        self.ee_block=i_block
        
        return True
        
    # -------------------------------------------------------------------
    # requires that you define _updateGuiSwing in subclasses !
    # @return true if the given i_block belongs to this gui
    def updateGuiFromBlock(self, block : EEblock ) -> bool:
        if not self._updateBlock(block):
            return False
        
        self.after_idle(self._updateGuiSwing)
        
        
    # -------------------------------------------------------------------
    # MUST be overriden by children
    # Update the block to be sent to radio getting data from the GUI
    # return true if the update is successful    
    def updateBlockFromGui(self) -> bool:
        return False
        
        
        

