'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.

Other parts of the system should send messages here to be sent
There has to be a class that manage the queue of incoming commands
this is because the GUI cannot do IO (will be come sluggish) and the serial channel is NON sharable by default

'''


from __future__ import annotations

from queue import Queue
import struct
from threading import Thread
import time
from tkinter import StringVar
import tkinter
import traceback

from app_config import AppConfig
from debug_gui import LogPanel
from glob_fun import getHomeFullPath
from glob_gui import TkToplevel, SolarCheckbox, TV_Entry 
from ipp_chunkid import IPPU_pollpending_reqid
import ipp_chunkid
from ipp_parser import Qsk5_res_RadioStatus, Qsk5_command, Ippparser, IppChunk,\
    Qsk5_res_Counters, Qsk5_res_Spectrum, Qsk5_res_Loudest, Qsk5_res_PollDtmf,\
    Qsk5_res_RadioPrintf, Qsk5_res_RadioIdentity, Qsk5_res_RadioConfiguration,\
    Qsk5_res_Generic, Qsk5_res_Eeprom
import tkinter.ttk as ttk
from uvk5_lib import Libuvk5serial


# ====================================================================
class QpypollThread (Thread):

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat'):
        Thread.__init__(self,name="Quansheng poll",daemon=True)
        
        self.stat = stat
        self.libk5 : Libuvk5serial = None;
        
        self._oqueue = Queue()
        
        self.stat.appconfig.addMyselfToSavables(self)

        self._toplevel = TkToplevel("Polling window")
        aframe=ttk.Frame(self._toplevel, padding=2 )     
    
        self._newReadWriteButtonsPanel(aframe).pack(fill='x')
        self._newConfigPanel(aframe).pack(fill='x')
        self._newStatusPanel(aframe).pack(fill='x')
        self._newRssiPanel(aframe).pack(fill='x',pady=2)  # this is how you put some padding when using pack
        
        self._poll_log = LogPanel(self.stat,aframe,"Commands Log")
        self._poll_log.setWrap(True);
        self._poll_log.pack(fill='both',expand=True)
        
        aframe.pack(fill='both',expand=True)
        
        self._println("Qppoll init complete")

    # -------------------------------------------------------------------
    # prints something on this window log
    def _println(self, msg):
        self._poll_log.println(msg)    
        
    # --------------------------------------------------------------------
    # show a window that has been hidden using the withdraw method
    def deiconify(self):
        self._toplevel.deiconify()    

    # --------------------------------------------------------------------
    # hide completely a window
    def withdraw(self):
        self._toplevel.withdraw()    

    # -----------------------------------------------------------
    # Mostly a reminder that this method is available everywhere    
    def _runOnGuiIdle(self, func, *args ):
        self._toplevel.after_idle(func, *args)

    # -------------------------------------------------------------------------------
    # make a new frame and put an input line in it
    def _newConfigPanel(self, parent):

        panel=ttk.Frame(parent,padding='3')
        
        ttk.Label(panel,text="Serial Port: ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self.serialPortName = StringVar(panel)
        
        TV_Entry(panel, self.serialPortName, width=30).pack(side='left')
        
        self._w_counters = SolarCheckbox(panel, "Counters");
        self._w_counters.pack(side='left')
        
        return panel

    # ----------------------------------------------------------------------------------
    # NOTE that there is style definition in _adjustTtkStyle
    def _newRssiPanel(self, parent):

        rssiFrame = ttk.Frame(parent, padding=3, borderwidth=1, relief='solid') 

        rssiFrame.columnconfigure(0, weight = 0)
        rssiFrame.columnconfigure(1, weight = 0) 
        rssiFrame.columnconfigure(2, weight = 1)   # since the other is zero, this will get all the extra space

        w_font = ('Consolas',10)
        w_font_bold = ('Consolas',10,'bold')

        rowIndex=0

        #RSSI
        ttk.Label(rssiFrame, font=w_font, text="RSSI dBm").grid(row = rowIndex, column = 0, sticky ="nsew")
        self.rssiLabel = ttk.Label(rssiFrame, font=w_font_bold, justify=tkinter.RIGHT)
        self.rssiLabel.grid(row = rowIndex, column = 1, sticky ="ew")
        self.rssiBar = ttk.Progressbar(rssiFrame, orient='horizontal', mode='determinate', maximum=0x1FF/2)
        self.rssiBar.grid(row = rowIndex, column = 2, sticky ="ew")
        rowIndex += 1

        #NOISE
        ttk.Label(rssiFrame, font=w_font, text="Ex-noise,  dB/step").grid(row = rowIndex, column = 0, sticky ="nsew")
        self.noiseLabel = ttk.Label(rssiFrame, font=w_font_bold, justify=tkinter.RIGHT)
        self.noiseLabel.grid(row = rowIndex, column = 1, sticky ="nsew")
        self.noiseBar = ttk.Progressbar(rssiFrame, orient='horizontal', mode='determinate', maximum=0x7F)
        self.noiseBar.grid(row = rowIndex, column = 2, sticky ="nsew")
        rowIndex += 1

        #GLITCH
        ttk.Label(rssiFrame, font=w_font, text="Glitch indicator   ").grid(row = rowIndex, column = 0, sticky ="nsew")
        self.glitchLabel = ttk.Label(rssiFrame, font=w_font_bold, justify=tkinter.LEFT)
        self.glitchLabel.grid(row = rowIndex, column = 1, sticky ="nsew")
        self.glitchBar = ttk.Progressbar(rssiFrame, orient='horizontal', mode='determinate', maximum=0xFF)
        self.glitchBar.grid(row = rowIndex, column = 2, sticky ="nsew")
        rowIndex += 1
        
        #BATTERY
        ttk.Label(rssiFrame, font=w_font, text="Battery Voltage").grid(row = rowIndex, column = 0, sticky ="nsew")
        self.battLabel = ttk.Label(rssiFrame, font=w_font_bold, justify=tkinter.LEFT)
        self.battLabel.grid(row = rowIndex, column = 1, sticky ="nsew")
        self.battBar = ttk.Progressbar(rssiFrame, orient='horizontal', mode='determinate', maximum=1000)
        self.battBar.grid(row = rowIndex, column = 2, sticky ="nsew")
        rowIndex += 1

        #CPU Temperature
        ttk.Label(rssiFrame, font=w_font, text="Temperature CPU").grid(row = rowIndex, column = 0, sticky ="nsew")
        self.tcpuLabel = ttk.Label(rssiFrame, font=w_font_bold, justify=tkinter.LEFT)
        self.tcpuLabel.grid(row = rowIndex, column = 1, sticky ="nsew")
        self.tcpuBar = ttk.Progressbar(rssiFrame, orient='horizontal', mode='determinate', maximum=4000)
        self.tcpuBar.grid(row = rowIndex, column = 2, sticky ="nsew")
        rowIndex += 1

        return rssiFrame 
    
    # ---------------------------------------------------------------------------------
    # this updates only the chunk with radio status
    def _updateRadioStatus(self, chunk : Qsk5_res_RadioStatus):
        
        rssi = chunk.rx_rssi / 2 - 160
        
        self.rssiLabel.config(text="{:7.2f}".format(rssi))
        self.rssiBar['value'] = rssi+160
        
        self.noiseLabel.config(text="{}".format(chunk.rx_noise))
        self.noiseBar['value'] = chunk.rx_noise
                
        self.glitchLabel.config(text="{}".format(chunk.rx_glitch))
        self.glitchBar['value'] = chunk.rx_glitch

        self.battLabel.config(text="{}".format(chunk.batteryV))
        self.battBar['value'] = chunk.batteryV_2dec

        self.tcpuLabel.config(text="{}".format(chunk.tc_cpu))
        self.tcpuBar['value'] = chunk.tc_cpu_2dec

        self.rxFrequency.config(text="RX Frequency HZ  {}".format(chunk.rxfreq))
        self.radioStatus.config(text="Radio Status bits  {:#04x}".format(chunk.radio_status))
    

    # -------------------------------------------------------------------------------
    # make a new frame and put an input line in it
    def _newStatusPanel(self, parent):

        panel=ttk.Frame(parent, padding=3,borderwidth=1, relief='solid')
        
        self.rxFrequency = ttk.Label(panel, justify=tkinter.LEFT,font=('calibri',14))
        self.rxFrequency.pack(side='top',fill='x',expand=True);
        
        self.radioStatus = ttk.Label(panel, justify=tkinter.LEFT,font=('calibri',14))
        self.radioStatus.pack(side='top',fill='x',expand=True);
        
        return panel


    # -------------------------------------------------------------------------------
    # make a new frame and put buttons into it, but stay in this class
    def _newReadWriteButtonsPanel(self, container):

        cframe = ttk.Frame(container, borderwidth=3, relief='ridge')

        ttk.Button(cframe,text="Save Log" , command=self._clicSaveLog ).pack(side='left')

        return cframe
    
    # --------------------------------------------------------------------------------------
    def _clicSaveLog(self):
        try:
            ffname = getHomeFullPath("qsk5gui.log")
            with open(ffname, 'w') as fileo:
                astring=self._poll_log.getTextString()
                fileo.writelines(astring)
                
                self._println('--')
                self._println("saved file name "+str(ffname))
                
        except Exception as exc :
            self._println("_clicSaveLog "+str(exc))


    # -------------------------------------------------------------------
    # implement the config savable
    def appImportConfig(self, cfg : AppConfig ):
        adict = cfg.getDictValue("qpypoll_cfg", {})
        
        try:
            self.serialPortName.set(adict['poll_port'])    
            self._toplevel.setGeometry(adict['gui_cfg'])
        except Exception as _exc :
            pass            

    # -------------------------------------------------------------------
    # implement the config savable
    def appSaveConfig(self, cfg : AppConfig ):
        adict = dict()

        adict['gui_cfg'] = self._toplevel.getGeometry()
        adict['poll_port'] = self.serialPortName.get()

        cfg.setDictValue("qpypoll_cfg", adict )            
    
    # ------------------------------------------------------
    # use this method to queue a command to be executed
    # it has to be a child of a command
    def queuePut (self,line : Qsk5_command):
        self._oqueue.put(line)
    
    
    # ------------------------------------------------------
    # Attempt to open the device and do some polling
    def _open_and_poll(self):
        try:
            portname=self.serialPortName.get()

            self._println("Qpypoll opening "+portname)

            if self.libk5:
                self.libk5.disconnect()
            else:
                self.libk5 = Libuvk5serial()

            self.libk5.connect(portname)
            
            while self._onePoll():
                time.sleep(0.05)
        
        except Exception as _err:
            self._println(traceback.format_exc())
    
    
    # --------------------------------------------------------------
    # Called by the Thread, do NOT call it, use start()
    # I wish this to stop on first error, so I can investigate
    def run(self):
        self._println ("START Thread "+self.name)
        
        while True:
            self._open_and_poll()
            self._println ("SLEEP forom poll error")
            time.sleep(5)

        self._println ("END Thread "+self.name)


    # --------------------------------------------------------------
    # return a byte array of a string padded to the given len
    def _pad_String(self, astring : str , pad_to : int ):
        ascii_bytes = bytes(astring,'ascii')
        
        return ascii_bytes + b'0x00' * (40 - len(ascii_bytes))
        
    
    
    # ------------------------------------------------------
    # Attempt to poll the queue of commands to send to quansheng
    # @return the Ippparser if there is something as result
    def _onePollQueue(self) -> Ippparser:
        
        if self._oqueue.empty():
            return False
        
        command : Qsk5_command = self._oqueue.get(True, 0.3)
        
        self.stat.app_println("_onePollQueue: have command "+str(command))
        
        return command.send_request(self.libk5)

    # ------------------------------------------------------
    '''
struct IPPU_req_polling
   {
   struct IPPU_header8 header;   // IPPU_req_pollpending

   uint16_t w_printf:1;        // bit0 poll for printf
   uint16_t w_ipp:1;           // bit1 poll for IPP packets
   uint16_t w_dtmfipp:1;    // bit2 poll for DTMF IPP packets
   uint16_t w_counters:1;  // bit3 poll for counters
   uint16_t w_dtmfrxtx:1;  // bit4 poll for DTMF
   uint16_t w_qsmsg:1;     // bit5 poll for Quansheng messages
   uint16_t w_scanloud:1;  // bit6 poll for scanloud result
   uint16_t padding:9;     // MSB on LE CPU
   };
    '''
    def _poll_pending(self):

        # also want spectrum
        w_mask = 0b11110001

        if self._w_counters.cvalBoolean:
            w_mask = w_mask | 0b1000
            self._w_counters.setChecked(False)

        # request everything
        s_chunk = struct.pack('<H',w_mask)
        
        buffer=self.libk5.ippp_send_chunk(IPPU_pollpending_reqid,s_chunk)
        
        return Ippparser(buffer) 
    


    # ------------------------------------------------------
    # use this method to queue a command to be executed
    def _onePoll(self) -> bool:
        
        try:
            # try to poll the queue of commands
            parser = self._onePollQueue()
            
            if not parser:
                # if nothing, then do a polling
                parser=self._poll_pending()

            # all replies have a different ID
            self._showChunks(parser)
            
        except Exception as _err:
            self._println(traceback.format_exc())
            return False
            
        return True


    # -------------------------------------------------------------
    def _showChunks(self, parser : Ippparser ):

        while parser.hasNextChunk():
            chunk=parser.getNextIppChunk()
            
            if chunk:
                self._showChunk(chunk)
        
    # -------------------------------------------------------------
    def _showChunk(self, chunk : IppChunk ):
        
        if chunk.chunk_id == ipp_chunkid.IPPU_pollprintf_resid:
            self._showRadioPrintf(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_pollradiostat_resid:
            self._runOnGuiIdle(self._updateRadioStatus,chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_pollqsmsg_resid:
            self.stat.qsmsg._println(str(chunk))
        elif chunk.chunk_id == ipp_chunkid.IPPU_ident_resid:
            self._showRadioIdentity(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_radiocfg_resid:
            self._showRadioConfig(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_generic_resid:
            self._showGenericResponse(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_polldtmf16_resid:
            self._showPollDtmfRes(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_scan_loudest_resid:
            self._showScanLoudestRes(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_res_counters16:
            self._showResCounters(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_res_spectrum16:
            self._showResSpectrum(chunk)
        elif chunk.chunk_id == ipp_chunkid.IPPU_res_eeprom:
            self._showResEeprom(chunk)
        else:
            self.stat.app_println("POLL unsupported chunkid="+str(chunk.chunk_id))
        
    # -------------------------------------------------------------
    # the result should be given to the Eeprom
    def _showResEeprom(self, chunk : Qsk5_res_Eeprom):
            #self._println("have EEPROM res address="+str(chunk.ee_rw_address)+' block='+str(chunk.ablock))
            self.stat.qseeprom.eepromUpdateFromRadio(chunk.ablock)


    
    # -------------------------------------------------------------
    # show in the main window the res counters result
    def _showResCounters(self, chunk : Qsk5_res_Counters):
            self._println("counters sw_version "+chunk.sw_version)
            self._println(" bfsk_rx_count="+str(chunk.bfsk_rx_count))
            self._println(" bfsk_tx_count="+str(chunk.bfsk_tx_count))
            
    # -------------------------------------------------------------
    # the result should be given to the spectrum window
    def _showResSpectrum(self, chunk : Qsk5_res_Spectrum):
            self._println("\nhave spectrum res F= "+str(chunk.start_freq_1x))
            self.stat.spectrum.showSpectrum_res(chunk)


    # -------------------------------------------------------------
    # show in the main window the identify result
    def _showScanLoudestRes(self, chunk : Qsk5_res_Loudest):
       
            self.stat.appmaingui.println("Scan Loudest="+str(chunk.found_F_1x))
        
        
    # -------------------------------------------------------------
    def _showPollDtmfRes(self, chunk : Qsk5_res_PollDtmf):
        self.stat.qsdtmf.dtmf_println(chunk.dtmf_string)
        
    # -------------------------------------------------------------
    def _showRadioPrintf(self, chunk : Qsk5_res_RadioPrintf):
        self._poll_log.write(chunk.print_this)
        
    # -------------------------------------------------------------
    # show in the main window the identify result
    def _showRadioIdentity(self, chunk : Qsk5_res_RadioIdentity):
            self.stat.appmaingui.println("HW model="+chunk.hw_model)
            self.stat.appmaingui.println("cpu id0="+'{:08X}'.format(chunk.id0))
            self.stat.appmaingui.println("cpu id1="+'{:08X}'.format(chunk.id1))
            self.stat.appmaingui.println("cpu id2="+'{:08X}'.format(chunk.id2))
            self.stat.appmaingui.println("cpu id3="+'{:08X}'.format(chunk.id3))
            self.stat.appmaingui.println("SW version="+chunk.sw_version)
    
    # -------------------------------------------------------------
    def _showRadioConfig(self, chunk : Qsk5_res_RadioConfiguration):
        self.stat.appmaingui.println("RADIO CONFIG RES:")
        self.stat.appmaingui.println("  rxf_1x "+str(chunk.rxfreq_1x))

    # -------------------------------------------------------------
    def _showGenericResponse(self, chunk : Qsk5_res_Generic):
        a_s = "GENERIC RES: oreqid "+str(chunk.from_reqid)+" reason="+str(chunk.reason)+" code="+str(chunk.rescode)+" message="+str(chunk.message)
        self.stat.appmaingui.println(a_s)





