'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''
# -------------------------------------------------------------------------
# There is only ONE poll, this is a GUI and if there is something to send it should be set to the poll

from __future__ import annotations

from tkinter import IntVar, StringVar

from app_config import AppConfig
from debug_gui import LogPanel 
from fmeetable import EEblock_0x0E40_FMtable_Gui
from glob_gui import TkToplevel, SolarCheckbox, TV_Entry, GridFrame, \
    EEblockGui 
from ipp_parser import \
    EEblock, Qsk5_req_eeprom, Qsk5_req_syscmd
import tkinter.ttk as ttk


# ====================================================================
class QseepromGui ():

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat'):
        self.stat = stat
        
        self.stat.appconfig.addMyselfToSavables(self)

        self._toplevel = TkToplevel("EEPROM window")
        
        aframe=ttk.Frame(self._toplevel, padding=2)     
    
        ttk.Label(aframe,text="First Read, then Write", font=("Courier", 24),anchor='center' ).pack(fill='x')
        self._newReadWriteButtonsPanel(aframe).pack(fill='x')
        self._newSubpanelsButtons(aframe).pack(fill='x')

        self._ee_blocks = []  # this will be the list of eeprom memory blocks
        
        self._newCenterPanel(aframe).pack(fill='both',expand=True)
        
        aframe.pack(fill='both',expand=True)
        
        self.eeprom_println("QseepromGui init complete")


    # --------------------------------------------------------------------
    # The center panel MUST be a grid layout where ALL subpanels are placed at 0,0
    # then, raise is used to show a specific one
    def _newCenterPanel(self, parentPanel ) -> ttk.Frame:
        
        apanel=ttk.Frame(parentPanel,padding='3')

        apanel.grid_rowconfigure(0, weight = 1)
        apanel.grid_columnconfigure(0, weight = 1)

        self._newLogPanel(apanel).grid(row=0, column=0, sticky ="nsew")
        self._newFMtablePanel(apanel).grid(row=0, column=0, sticky ="nsew")
        self._newParametersPanel(apanel).grid(row=0, column=0, sticky ="nsew")
        
        return apanel
        

    # -------------------------------------------------------------------------------
    # Need to leave like this since grid does not return itself
    def _newParametersPanel(self, parentPanel) -> ttk.Frame:
        self._parametersPanel=ttk.Frame(parentPanel)
        
        self._addToGui(EEblock_0x0F40_Gui(self.stat,self._parametersPanel))
        self._addToGui(EEblock_0x0E70_Gui(self.stat,self._parametersPanel))
        self._addToGui(EEblock_0x0EA0_Gui(self.stat,self._parametersPanel))
        self._addToGui(EEblock_0x0E80_Gui(self.stat,self._parametersPanel))
        self._addToGui(EEblock_0x0F20_Gui(self.stat,self._parametersPanel))
        self._addToGui(EEblock_0x0EB0_Gui(self.stat,self._parametersPanel))
        self._addToGui(EEblock_0x0EC0_Gui(self.stat,self._parametersPanel))
        
        return self._parametersPanel 
        
    
    # -------------------------------------------------------------------------------
    # Need to leave like this since grid does not return itself
    def _newFMtablePanel(self, parentPanel) -> ttk.Frame:
        self._ee_fmtable=EEblock_0x0E40_FMtable_Gui(self.stat,parentPanel)
        
        self._ee_blocks.append(self._ee_fmtable)
        
        return self._ee_fmtable
        
    # -------------------------------------------------------------------------------
    # Need to leave like this since grid does not return itself
    def _newLogPanel(self, parentPanel ) -> ttk.Frame:
        self._log_gui = LogPanel(self.stat,parentPanel,"EEPROM Log")
        self._log_gui.setWrap(True);
        self._log_gui.pack(fill='both',expand=True)
        
        return self._log_gui 
        
        
    # -------------------------------------------------------------------------------
    # I need buttons for the subpanels of this class
    def _newSubpanelsButtons(self, parentPanel):

        cframe = ttk.Frame(parentPanel)

        ttk.Button(cframe,text="Parameters" , command=self._clicShowParameters ).pack(side='left',padx=3)
        ttk.Button(cframe,text="FM Channels" , command=self._clicShowFmchannels).pack(side='left',padx=3)
        ttk.Button(cframe,text="Log Panel" , command=self._clicShowLog).pack(side='left',padx=3)

        return cframe

    def _clicShowParameters(self):
        self._parametersPanel.tkraise()

    def _clicShowFmchannels(self):
        self._ee_fmtable.tkraise()

    def _clicShowLog(self):
        self._log_gui.tkraise()



    # --------------------------------------------------------------------
    def _addToGui(self, ablock : EEblockGui ):
        self._ee_blocks.append(ablock)
        ablock.pack(fill='x', pady=3)
        
    # -------------------------------------------------------------------
    # prints something on the application log
    def eeprom_println(self, msg):
        self._log_gui.println(msg)    
        
    # --------------------------------------------------------------------
    # show a window that has been hidden using the withdraw method
    def deiconify(self):
        self._toplevel.deiconify()    

    # --------------------------------------------------------------------
    # hide completely a window
    def withdraw(self):
        self._toplevel.withdraw()    

    # -----------------------------------------------------------
    # Mostly a reminder that this method is available everywhere    
    def _runOnGuiIdle(self, func, *args ):
        self.frame.after_idle(func, *args)

    # -------------------------------------------------------------------------------
    # make a new frame and put buttons into it, but stay in this class
    def _newReadWriteButtonsPanel(self, container):
        #cframe = ttk.Frame(container, borderwidth=3, relief='ridge')
        cframe = ttk.Frame(container)

        ttk.Button(cframe,text="Read" , command=self._clicReadEeprom ).pack(side='left',padx=3)
        ttk.Button(cframe,text="Write" , command=self._clicWriteEeprom).pack(side='left',padx=3)
        
        self._en_write_reboot = SolarCheckbox(cframe,"EN Reboot on Write")
        self._en_write_reboot.pack(side='left',padx=3)

        self._en_write_reload = SolarCheckbox(cframe,"EN Reload on Write")
        self._en_write_reload.pack(side='left',padx=3)

        return cframe
    


    
    # --------------------------------------------------------------------------------------
    # Every block will put its own request, if possible
    def _clicWriteEeprom(self):
        self.eeprom_println("CLICK Write Eeprom")

        for arow in self._ee_blocks:
            ablockGui : EEblockGui = arow
            
            if ablockGui.updateBlockFromGui():
                ee_start = ablockGui.ee_block.ee_start_a
                ee_len = ablockGui.ee_block.ee_len
                ee_bytearray =  ablockGui.ee_block.ee_bytearray
                
                cmd = Qsk5_req_eeprom(ee_start, ee_len, ee_bytearray)
                self.stat.qpypoll.queuePut(cmd)

        if self._en_write_reboot.isChecked(): 
            cmd = Qsk5_req_syscmd(1)
            self.stat.qpypoll.queuePut(cmd)
        elif self._en_write_reload.isChecked():
            cmd = Qsk5_req_syscmd(2)
            self.stat.qpypoll.queuePut(cmd)

    # --------------------------------------------------------------------------------------
    # I need to make it a command that goes to the polling window
    # Polling, will eventually call me back with the data
    def _clicReadEeprom(self):
        self.eeprom_println("CLICK Read Eeprom")

        for arow in self._ee_blocks:
            ablockGui : EEblockGui = arow
            s_Addr = ablockGui.ee_block.ee_start_a
            s_len =ablockGui.ee_block.ee_len
            cmd = Qsk5_req_eeprom(s_Addr,s_len,None)
            self.stat.qpypoll.queuePut(cmd)
        

    # -------------------------------------------------------------------
    # implement the config savable
    def appImportConfig(self, cfg : AppConfig ):
        adict = cfg.getDictValue("qseeprom_cfg", {})
        
        try:
            self._toplevel.setGeometry(adict['gui_cfg'])
            self._en_write_reboot.setChecked(adict['en_write_reboot'])
            self._en_write_reload.setChecked(adict['en_write_reload'])
        except Exception as _exc :
            pass            

    # -------------------------------------------------------------------
    # implement the config savable
    def appSaveConfig(self, cfg : AppConfig ):
        adict = dict()

        adict['gui_cfg'] = self._toplevel.getGeometry()
        adict['en_write_reboot'] = self._en_write_reboot.isChecked()
        adict['en_write_reload'] = self._en_write_reload.isChecked()

        cfg.setDictValue("qseeprom_cfg", adict )            
    
    
    # ----------------------------------------------------------------------
    # this will receive a block and attempt a gui update
    # this is NOT called from a swing thread, so, it must queue
    def eepromUpdateFromRadio (self, block : EEblock ):

        if not block:
            return

        for arow in self._ee_blocks:
            ablock : EEblockGui = arow
            
            if ( ablock.updateGuiFromBlock(block)):
                break
        
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    


    
    
        
# ========================================================================
# so, the generic gui should require the start address
        
class EEblock_0x0F40_Gui (EEblockGui):

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self,parent_frame, 0xF40, 8)
        
        self.stat = stat

        self.c_panel=GridFrame(self,padding='3')

        self.c_panel.grid_rowconfigure(0, weight = 1)
        self.c_panel.grid_columnconfigure(1, weight = 1, pad=1)
        
        self._en_pmr = SolarCheckbox(self.c_panel,"EN PMR TX")
        self.c_panel.addItem(self._en_pmr)

        self._en_memch_wr = SolarCheckbox(self.c_panel,"MEMCH WR")
        self.c_panel.addItem(self._en_memch_wr)

        self._en_always_home = SolarCheckbox(self.c_panel,"ALWAYS HOME")
        self.c_panel.addItem(self._en_always_home)

        self._en_FILTER_M = SolarCheckbox(self.c_panel,"FILTER MSGS")
        self.c_panel.addItem(self._en_FILTER_M)

        self._en_FL_BLIP = SolarCheckbox(self.c_panel,"FLASHL BLIP")
        self.c_panel.addItem(self._en_FL_BLIP)
        
        self._en_hambands = SolarCheckbox(self.c_panel,"EN HAM TX")
        self.c_panel.addItem(self._en_hambands)

        self.c_panel.nextRow()

        self._en_MIC_agc = SolarCheckbox(self.c_panel,"MIC AGC")
        self.c_panel.addItem(self._en_MIC_agc)

        self._en_AF_VOXc = SolarCheckbox(self.c_panel,"AF VOX")
        self.c_panel.addItem(self._en_AF_VOXc)

        self._en_BFSK_SYLED = SolarCheckbox(self.c_panel,"FSK SYLED")
        self.c_panel.addItem(self._en_BFSK_SYLED)

        self._KEYS_locked = SolarCheckbox(self.c_panel,"KEYS lock")
        self.c_panel.addItem(self._KEYS_locked)

        #damiano

        self.c_panel.pack(side='left')

    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):
        
        abyte = self.ee_block.get_byte(1)

        self._en_pmr.setChecked(        (abyte & 0b00000001 ) != 0)
        self._en_memch_wr.setChecked(   (abyte & 0b00000010 ) != 0)
        self._en_always_home.setChecked((abyte & 0b00000100 ) != 0)
        self._en_FILTER_M.setChecked(   (abyte & 0b00001000 ) != 0)
        self._en_FL_BLIP.setChecked(    (abyte & 0b00010000 ) != 0)
        self._en_hambands.setChecked(   (abyte & 0b00100000 ) != 0)
    
        abyte = self.ee_block.get_byte(2)

        self._en_MIC_agc.setChecked(   (abyte & 0b00000001 ) != 0)
        self._en_AF_VOXc.setChecked(   (abyte & 0b00000010 ) != 0)
        self._en_BFSK_SYLED.setChecked((abyte & 0b00000100 ) != 0)
        self._KEYS_locked.setChecked(  (abyte & 0b00001000 ) != 0)
        

    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if not self.ee_block.ee_bytearray:
            return False
        
        bbuf = self.ee_block.ee_bytearray

        if len(bbuf) != self.ee_block.ee_len:
            return False
        
        abyte = 0; # initial value

        if self._en_pmr.cvalBoolean:
            abyte |= 0b00000001
            
        if self._en_memch_wr.cvalBoolean:
            abyte |= 0b00000010

        if self._en_always_home.cvalBoolean:
            abyte |= 0b00000100

        if self._en_FILTER_M.cvalBoolean:
            abyte |= 0b00001000

        if self._en_FL_BLIP.cvalBoolean:
            abyte |= 0b00010000

        if self._en_hambands.cvalBoolean:
            abyte |= 0b00100000

        self.ee_block.set_byte(1,abyte)

        # ----------------------- another byte

        abyte = 0; 

        if self._en_MIC_agc.cvalBoolean:
            abyte |= 0b00000001

        if self._en_AF_VOXc.cvalBoolean:
            abyte |= 0b00000010

        if self._en_BFSK_SYLED.cvalBoolean:
            abyte |= 0b00000100

        if self._KEYS_locked.cvalBoolean:
            abyte |= 0b00001000

        self.ee_block.set_byte(2,abyte)
        
        return True
        
        



# ========================================================================
# mostly to fix the TAIL len
        
class EEblock_0x0EA0_Gui (EEblockGui):

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self, parent_frame, 0x0EA0, 16)
        
        self.stat = stat
        
        self.c_panel=GridFrame(self,padding='3')

        self._vox_DDelayTime    = IntVar(self.c_panel,value=8)
        self._vox_DIntervalTime = IntVar(self.c_panel,value=8)
        self._msg_from          = IntVar(self.c_panel,value=22)
        self._tail_tone_len     = IntVar(self.c_panel,value=30)
        self._msg_ack           = SolarCheckbox(self.c_panel,'on/off')
        self._set_defaults      = SolarCheckbox(self.c_panel,'on/off')

        self.c_panel.grid_rowconfigure(0, weight = 1)
        self.c_panel.grid_columnconfigure(1, weight = 1, pad=1)
        
        self.c_panel.addItem(ttk.Label(self.c_panel,text="VOX DD Time"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="VOX DI Time"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="MSG_FROM"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="TAILT len cs"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="MSG ACK"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="Set defaults"))
        
        self.c_panel.nextRow()

        self.c_panel.addItem(TV_Entry(self.c_panel, self._vox_DDelayTime ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._vox_DIntervalTime ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._msg_from      ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._tail_tone_len ,width=4))
        self.c_panel.addItem(self._msg_ack)
        self.c_panel.addItem(self._set_defaults)
        
        self.c_panel.pack(side='left')
        
        

    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):

        self._vox_DDelayTime.set(self.ee_block.get_byte(0))
        self._msg_from.set(self.ee_block.get_byte(4))
        self._vox_DIntervalTime.set(self.ee_block.get_byte(6))
        self._tail_tone_len.set(self.ee_block.get_byte(10))
    
        abyte = self.ee_block.get_byte(3)
        
        self._msg_ack.setChecked(     (abyte & 0b00000010 ) != 0)

        abyte = self.ee_block.get_byte(15)
        
        self._set_defaults.setChecked( (abyte != 1) )
    

    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if not self.ee_block.ee_bytearray:
            return False
        
        bbuf = self.ee_block.ee_bytearray

        if len(bbuf) != self.ee_block.ee_len:
            return False

        self.ee_block.set_byte(0,self._vox_DDelayTime.get())
        self.ee_block.set_byte(4,self._msg_from.get())
        self.ee_block.set_byte(6,self._vox_DIntervalTime.get())
        self.ee_block.set_byte(10,self._tail_tone_len.get())
        
        # ----------------------------------- this is a bitfield inside a byte
        
        abyte = self.ee_block.get_byte(3) 

        if self._msg_ack.cvalBoolean:
            abyte |= 0b00000010
        else:
            abyte &= 0b11111101
            
        self.ee_block.set_byte(3,abyte)    
        
        # ----------------------------------- this is instead a single value 

        if self._set_defaults.cvalBoolean:
            self.ee_block.set_byte(15,0)    
        
        # ----------------------------------- 

        
        return True
        







# ========================================================================
# I am picking up 16 bytes, since it is possible
        
class EEblock_0x0E70_Gui (EEblockGui):
    
    '''
    VOX logic, not easy
    1) There is an ENABLE VOX bit that is used to enable the whole mechanism
       VOX can be enable/disabled by F - 7 in the main screen
       can also be enabled using PerpperDock, there is a checkbox
       
    2) VOX depends on the MIC level, and there is 0xE70 MIC SENS idx from 0 to 4 that adjust the mic sensitivity
       However, it is not clear if the VOX levels are bound to this sensisivity or not
       
    3) There is a VOX voice level, from 1 to 100 that should define the level at which the "voice" is detected
       by default the initial chip value has 8 as a value
    
    4) There is a VOX silcence level, from 1 to 100 that defines when silence is detected
       by default the initial chip value is 6 as a value
       
    5) There is a VOX Detection Delay, in units of 128ms, initial value 8 and max value 15
       It seems like that the vox status is reported with a delay
    
    6) There is a VOX Detection Interval, possibly in units of 128ms, initial value 8 and max value 15
       Meaning that this time interval is vaited to decide what to report
       
     7) There is a VOX Hunkhup delay time (units of tents of second) that define the time to wait before switch to RX since vox detected silence.
     
     8) VOX should NOT kill a TX that has started by someone else (PTT, DTMF, or others) and you can see the TX source in the main panel
        The label S1 will appear if you press PTT, when using vox S4 will appear
        
     9) It is possible to see VOX reactions in real time, in the status bar, when vox detect voice a >< symbol will appear
        It will disappear when detect silcence    
        
     A user should have all information to adjust VOX to suit the needs    
     
     NOTE: MIC gain is now a direct value into the chip, meaning that it has the following range
     0=min; 0x1F=max; 0.5dB/step initial value 0x10
     
     
     
        
    '''
    

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self,parent_frame, 0x0E70, 16)
        
        self.stat = stat

        self.c_panel=GridFrame(self,padding='3')

        self._homech       = IntVar(self.c_panel,value=0)
        self._vox_en_lvl   = IntVar(self.c_panel,value=2)
        self._vox_dis_lvl  = IntVar(self.c_panel,value=1)
        self._mic_gain     = IntVar(self.c_panel,value=0x10)
        self._vox_minT_ds  = IntVar(self.c_panel,value=20)   # VOX minimum time in tenth of seconds

        self.c_panel.grid_rowconfigure(0, weight = 1)
        self.c_panel.grid_columnconfigure(1, weight = 1, pad=1)
        

        self.c_panel.addItem(ttk.Label(self.c_panel,text="Home CH"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="VOX voice lvl"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="VOX sil. lvl"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="MIC gain"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="VOX min Time ds"))
        
        self.c_panel.nextRow()

        self.c_panel.addItem(TV_Entry(self.c_panel, self._homech ,width=4     ))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._vox_en_lvl  ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._vox_dis_lvl ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._mic_gain,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._vox_minT_ds ,width=4))

        self.c_panel.pack(side='left')


    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):
        
        self._homech.set(self.ee_block.get_byte(0))
        self._vox_dis_lvl.set(self.ee_block.get_byte(5))
        self._vox_en_lvl.set(self.ee_block.get_byte(6))
        self._mic_gain.set(self.ee_block.get_byte(7))
        self._vox_minT_ds.set(self.ee_block.get_byte(14))


    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if self.ee_block.get_data_len() != self.ee_block.ee_len:
            return False

        self.ee_block.set_byte(0, self._homech.get())
        self.ee_block.set_byte(5, self._vox_dis_lvl.get())
        self.ee_block.set_byte(6, self._vox_en_lvl.get())
        self.ee_block.set_byte(7, self._mic_gain.get())
        self.ee_block.set_byte(14, self._vox_minT_ds.get())
        
        return True































# ========================================================================
# I am picking up 16 bytes, since it is possible
        
class EEblock_0x0E80_Gui (EEblockGui):
    
    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self,parent_frame, 0x0E80, 16)
        
        self.stat = stat

        self.c_panel=GridFrame(self,padding='3')

        self._ledw2_on_ts  = IntVar(self.c_panel,value=1)
        self._ledw2_off_ts = IntVar(self.c_panel,value=1)
        
        self._scl_noaudio_cmax = IntVar(self.c_panel,value=5)
        self._scl_withaudio_cmax = IntVar(self.c_panel,value=100)
        

        self.c_panel.grid_rowconfigure(0, weight = 1)
        self.c_panel.grid_columnconfigure(1, weight = 1, pad=1)
        
        self.c_panel.addItem(ttk.Label(self.c_panel,text="LEDW_2_on_ts"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="LEDW_2_off_ts"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="Scanlist noaudio cmax"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="scanlist with audio cmax"))
        
        self.c_panel.nextRow()

        self.c_panel.addItem(TV_Entry(self.c_panel, self._ledw2_on_ts   ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._ledw2_off_ts  ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._scl_noaudio_cmax  ,width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, self._scl_withaudio_cmax  ,width=4))

        self.c_panel.pack(side='left')


    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):
        
        self._ledw2_on_ts.set(self.ee_block.get_byte(8))
        self._ledw2_off_ts.set(self.ee_block.get_byte(9))
        
        self._scl_noaudio_cmax.set(self.ee_block.get_byte(12))
        self._scl_withaudio_cmax.set(self.ee_block.get_byte(13))


    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if self.ee_block.get_data_len() != self.ee_block.ee_len:
            return False

        self.ee_block.set_byte(8, self._ledw2_on_ts.get())
        self.ee_block.set_byte(9, self._ledw2_off_ts.get())
        
        self.ee_block.set_byte(12, self._scl_noaudio_cmax.get())
        self.ee_block.set_byte(13, self._scl_withaudio_cmax.get())

        return True

























# ========================================================================
       
class EEblock_0x0EB0_Gui (EEblockGui):
    
    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self,parent_frame, 0x0EB0, 16)
        
        self.stat = stat

        self.c_panel=ttk.Frame(self,padding='3')

        self._astring  = StringVar(self.c_panel)
        
        ttk.Label(self.c_panel,text="Welcome1 ").pack(side='left')

        TV_Entry(self.c_panel, self._astring ).pack(side='left', fill='x')

        self.c_panel.pack(side='left', fill='x')
        

    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):
        
        self._astring.set(self.ee_block.get_string())


    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if self.ee_block.get_data_len() != self.ee_block.ee_len:
            return False

        self.ee_block.set_string(self._astring.get())

        return True












# ========================================================================
       
class EEblock_0x0EC0_Gui (EEblockGui):
    
    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self,parent_frame, 0x0EC0, 16)
        
        self.stat = stat

        self.c_panel=ttk.Frame(self,padding='3')

        self._astring  = StringVar(self.c_panel)
        
        ttk.Label(self.c_panel,text="Welcome2 ").pack(side='left')

        TV_Entry(self.c_panel, self._astring ).pack(side='left', fill='x')

        self.c_panel.pack(side='left', fill='x')
        

    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):
        
        self._astring.set(self.ee_block.get_string())


    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if self.ee_block.get_data_len() != self.ee_block.ee_len:
            return False

        self.ee_block.set_string(self._astring.get())

        return True


























































































# ========================================================================
# so, the generic gui should require the start address
        
class EEblock_0x0F20_Gui (EEblockGui):

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self,parent_frame, 0x0F20, 8)
        
        self.stat = stat

        self.c_panel=GridFrame(self,padding='3')

        self._bs1_off = IntVar(self.c_panel,value='2')
        self._bs1_on = IntVar(self.c_panel,value='1')
        
        self._bs2_off = IntVar(self.c_panel,value='4')
        self._bs2_on = IntVar(self.c_panel,value='3')

        self._bs3_off = IntVar(self.c_panel,value='6')
        self._bs3_on = IntVar(self.c_panel,value='5')

        self._bs4_off = IntVar(self.c_panel,value='8')
        self._bs4_on = IntVar(self.c_panel,value='7')

        self.c_panel.grid_rowconfigure(0, weight = 1)
        self.c_panel.grid_columnconfigure(1, weight = 1)

        self.c_panel.addItem(ttk.Label(self.c_panel,text="BATT SAVE"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="Level 1"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="Level 2"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="Level 3"))
        self.c_panel.addItem(ttk.Label(self.c_panel,text="Level 4"))
        
        self.c_panel.nextRow()
        self.add_one_row("OFF cs",  self._bs1_off, self._bs2_off, self._bs3_off, self._bs4_off)

        self.c_panel.nextRow()
        self.add_one_row("ON cs",  self._bs1_on, self._bs2_on, self._bs3_on, self._bs4_on)

        self.c_panel.pack(side='left')


    def add_one_row(self, label, comp1, comp2, comp3, comp4 ):
        
        self.c_panel.addItem(ttk.Label(self.c_panel,text=label))

        self.c_panel.addItem(TV_Entry(self.c_panel, comp1, width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, comp2, width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, comp3, width=4))
        self.c_panel.addItem(TV_Entry(self.c_panel, comp4, width=4))

    # ----------------------------------------------------------------
    # this MUST be called in swing thread
    def _updateGuiSwing(self):
        
        self._bs1_off.set(self.ee_block.get_byte(0))
        self._bs1_on.set(self.ee_block.get_byte(1))

        self._bs2_off.set(self.ee_block.get_byte(2))
        self._bs2_on.set(self.ee_block.get_byte(3))

        self._bs3_off.set(self.ee_block.get_byte(4))
        self._bs3_on.set(self.ee_block.get_byte(5))
    
        self._bs4_off.set(self.ee_block.get_byte(6))
        self._bs4_on.set(self.ee_block.get_byte(7))


    # -----------------------------------------------------------------
    # whould tabe the gui and update the block
    # you need to have read the memory, first...
    def updateBlockFromGui(self) -> bool:
        
        if self.ee_block.get_data_len() != self.ee_block.ee_len:
            return False

        self.ee_block.set_byte(0, self._bs1_off.get())
        self.ee_block.set_byte(1, self._bs1_on.get())

        self.ee_block.set_byte(2, self._bs2_off.get())
        self.ee_block.set_byte(3, self._bs2_on.get())

        self.ee_block.set_byte(4, self._bs3_off.get())
        self.ee_block.set_byte(5, self._bs3_on.get())

        self.ee_block.set_byte(6, self._bs4_off.get())
        self.ee_block.set_byte(7, self._bs4_on.get())
        
        return True
        
        


































































