'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
 
 
'''
# the idea is that this allocate the main console in a GUI
# You can have the standard console in the "main thread", this is one in a GUI
# I need a model similar to Java, for reliability, so, NO variables in module
# python has a weird idea on the scope of a module variable from other modules
# Plus, no private keyword, at all
# TO have this updated on save you need to configure the editor Save Actions
# ALSO the INITIAL VALUE MUST be valid for the given format eg: yyyy-MM-dd HH:mm 
# __updated__ = "2022-07-25 09:19"

from __future__ import annotations

import os
from threading import Thread
from tkinter import ttk

from app_config import AppConfig
from app_gui import InputParserThread
import app_gui 
from debug_gui import DbglsGui 
from glob_gui import  GuiRoot
from poll_gui import QpypollThread
from qsdtmf_gui import QsdtmfGui
from qseeprom_gui import QseepromGui
from qsmsg_gui import QsmsgGui
from spectrum_gui import SpectrumGui
import tkinter as tk


__updated__ = "2024-10-21 10:32" 


# -----------------------------------------------------------------
# It has to be a thread since the main gui loop has to be in a thread
# NOTE that the remaining system setup IS in the gui run() since that is the ONLY way to make sure things are setup
# in the correct order
# do NOT set the thread to daemon, it will mess up with the input queue
class NewAppGuiThread (Thread):

    font_size=14


    # ----------------------------------------------------------------------
    # Constructor
    def __init__(self, stat : 'Qpystat'):
        Thread.__init__(self,name="PrepperDock GUI Thread")

        self.stat=stat
        
        # add myself to the pieces that can be saved and restored
        self.stat.appconfig.addMyselfToSavables(self)
        
        # this first, since the GUI need to know the input queue
        self.stat.inputParser = InputParserThread(stat)


    # ----------------------------------------------------------------------
    # This is the new thread, so, we can allocate the main Gui and ALL the remaining component
    def run(self):
        print ("START Thread ", self.name)
        
        self.root = GuiRoot(self.stat, "PrepperDock Controller", 'prepperdock', self.on_closing, False)
        
        self.menu = QuanshengMenu(self.root,self.stat)
        self.root.config(menu=self.menu)
        
        self._adjustTtkStyle()

        self.stat.dbgls = DbglsGui(self.stat)
        
        # The magic of tkraise works ONLY with the grid layout
        # the idea is that you sticky ="nsew" and put all Frames at 0,0
        # then, you can raise them to the top with tkraise
        self.root.grid_rowconfigure(0, weight = 1)
        self.root.grid_columnconfigure(0, weight = 1)
        
        self.stat.appmaingui = app_gui.AppMainGui(self.root,self.stat)
        self.stat.appmaingui.grid(row = 0, column = 0, sticky ="nsew")

        # ---------------------- end of gui elements that are switchable ---------------------
        
        # this will manage the debug configuration GUI
        self.stat.qpypoll  = QpypollThread(self.stat)
        self.stat.qsmsg    = QsmsgGui(self.stat)
        self.stat.qsdtmf   = QsdtmfGui(self.stat)
        self.stat.qseeprom = QseepromGui(self.stat)
        self.stat.spectrum = SpectrumGui(self.stat)

        # show the commands panel
        self.raiseCommands()
        
        # this will run half second later, to adjust windows properties and start servers
        self.root.runLater(500,self._runLaterOnsettled)

        # this is basically the Swing events thread
        self.root.mainloop()

        # NOTE that you MUST save window position when they are still active
        self.stat.appconfig.saveConfigToFile()    

        print ("END Thread ", self.name)
        
        # This is the most reliable way to terminate all the stuff
        os._exit(0)


    # -----------------------------------------------------------------
    # this is what should be done, on this window closing        
    def on_closing(self):
        self.stat.inputParser.queuePut('quit')

    # --------------------------------------------------------------------
    # See https://tkdocs.com/tutorial/styles.html
    # AH, remember that ALL widgets styles begin with T
    # AH, remember that ttk.Entry font CANNOT be specified using a style ....
    def _adjustTtkStyle(self):
        style = ttk.Style()
        style.configure("Treeview", highlightthickness=0, bd=0, font=('Calibri', self.font_size)) # Modify the font of the body
        style.configure("Treeview.Heading", font=('Calibri', self.font_size,'bold')) # Modify the font of the headings
        style.configure("TEntry", foreground = 'green', font=('Calibri', 13,'bold'), width=4) # Modify the font of the headings
        style.configure("TProgressbar", thickness=10)

    # --------------------------------------------------------------------
    # implement the config savable
    def appImportConfig(self, cfg : AppConfig ):
        self.root.setGeometry(cfg.getDictValue("pbxsrv_gui", {}))

    # --------------------------------------------------------------------
    # implement the config savable
    def appSaveConfig(self, cfg : AppConfig ):
        cfg.setDictValue("pbxsrv_gui", self.root.getGeometry())
        
    # ----------------------------------------------------------------
    # This is called when the GUI is up and running, basically, started on gui thread
    # with a delay of 500ms, needed to set the window status appropriately
    def _runLaterOnsettled(self):
        self.stat.appconfig.importPartConfig()
        
        # thread have to start once the GUI is up and running
        self.stat.inputParser.start()
        self.stat.qpypoll.start()
        
        self.stat.app_println("server "+__updated__+" started")


        
    # -------------------------------------------------------------------
    # called by another thread to shutdown the GUI
    # before shutting down, save the window position
    def endGuiConsole(self):
#        print ("endGuiConsole CALL")

        # window position must be saved before stopping GUI
        self.stat.appconfig.savePartConfig()

        # this will terminate the event loop
        # MUST be run on the gui thread to avoid concurrency issues 
        self.root.runIdle(self.root.destroy)

    # ------------------------------------------------------
    # Example on how you can raise different panels to top        
    def raiseCommands (self):
        self.stat.appmaingui.tkraise()


            
        
# ===========================================================================        
# Want to wrap the whole menu mess in here
# This part of the gui must do nothing until all pieces are actually in place

class QuanshengMenu(tk.Menu):

    # ------------------------------------------------------
    def __init__(self, guiroot, stat : 'Qpystat'):
        tk.Menu.__init__(self,guiroot)
        
        self.stat=stat;
        
        menutools = tk.Menu(self,tearoff=0)
        
        menutools.add_command(label="Show Console", command=self.showConsole)
    
        self.add_cascade(label="File", menu=menutools)

    # ------------------------------------------------------
    def showConsole(self):
        self.stat.app_println("show Console")
        self.stat.appguithread.raiseCommands()


    
    
