'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''
# -------------------------------------------------------------------------
# There is only ONE poll, this is a GUI and if there is something to send it should be set to the poll

from __future__ import annotations

from tkinter import IntVar

from app_config import AppConfig
from debug_gui import LogPanel 
from glob_gui import TkToplevel 
from ipp_parser import Qsk5_req_spectrum_scan, Qsk5_res_Spectrum
import tkinter.ttk as ttk


class SpectrumGui ():

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat'):
        self.stat = stat
        
        self.stat.appconfig.addMyselfToSavables(self)

        self._toplevel = TkToplevel("Spectrum window")
        
        aframe=ttk.Frame(self._toplevel, padding=2)     
    
        self.cmdsbuttons = self._newReadWriteButtonsPanel(aframe)
        self.cmdsbuttons.pack(fill='x')
        
        self._newInputPanel(aframe).pack(fill='x')
        
        self.qsdtmf_log = LogPanel(self.stat,aframe,"Spectrum Log")
        self.qsdtmf_log.setWrap(True);
        self.qsdtmf_log.pack(fill='both',expand=True)
        
        aframe.pack(fill='both',expand=True)
        
        self._spectrum_println("SpectrumGui init complete")

    # -------------------------------------------------------------------
    # prints something on the application log
    def _spectrum_println(self, msg):
        self.qsdtmf_log.println(msg)    
        
    # -------------------------------------------------------------------
    # Shows the values, now
    def showSpectrum_res(self, chunk : Qsk5_res_Spectrum):
        
        cur_F = chunk.start_freq_1x;
        step_F = chunk.step_freq_1x;
        
        self._spectrum_println("start F_1x "+str(cur_F))
                
        s_count = chunk.samples_count;
        
        for index in range(s_count):
            rssi_v = chunk.samples[index]
            self._spectrum_println("["+str(index)+"] F="+str(cur_F)+" rssi="+str(rssi_v))

            # increment after display
            cur_F = cur_F + step_F
            
        
    # --------------------------------------------------------------------
    # show a window that has been hidden using the withdraw method
    def deiconify(self):
        self._toplevel.deiconify()    

    # --------------------------------------------------------------------
    # hide completely a window
    def withdraw(self):
        self._toplevel.withdraw()    

    # -----------------------------------------------------------
    # Mostly a reminder that this method is available everywhere    
    def _runOnGuiIdle(self, func, *args ):
        self.frame.after_idle(func, *args)

    
    
    # -------------------------------------------------------------------------------
    # Example of how to use grid layout
    def _newInputPanel(self,parent):
        
        panel=ttk.Frame(parent,padding='3')

        self._edit_start_frequency = IntVar(panel,value='44600000')
        self._edit_step_F_1x = IntVar(panel,value='10000')
        self._edit_samples_count = IntVar(panel,value='10')
        self._edit_stime_cs = IntVar(panel,value='2')

#        panel.grid_rowconfigure(0, weight = 1)
        panel.grid_columnconfigure(1, weight = 1)
        
        row_index=0
        gstic="nsew"
        
        ttk.Label(panel,text="Start Frequency ").grid(row = row_index, column = 0, sticky =gstic)
        ttk.Entry(panel, font=('calibri',13), textvariable=self._edit_start_frequency).grid(row = row_index, column = 1, sticky =gstic)
        ttk.Label(panel,text="10Hz").grid(row = row_index, column = 2, sticky =gstic)

        row_index = row_index + 1
        
        ttk.Label(panel,text="Step frequency 1x").grid(row = row_index, column = 0, sticky =gstic)
        ttk.Entry(panel, font=('calibri',13), textvariable=self._edit_step_F_1x).grid(row = row_index, column = 1, sticky =gstic)
        ttk.Label(panel,text="10Hz").grid(row = row_index, column = 2, sticky =gstic)
        
        row_index = row_index + 1
        
        ttk.Label(panel,text="Samples Count ").grid(row = row_index, column = 0, sticky =gstic)
        ttk.Entry(panel, font=('calibri',13), textvariable=self._edit_samples_count).grid(row = row_index, column = 1, sticky =gstic)
        
        row_index = row_index + 1
        
        ttk.Label(panel,text="Sample time ").grid(row = row_index, column = 0, sticky =gstic)
        ttk.Entry(panel, font=('calibri',13), textvariable=self._edit_stime_cs).grid(row = row_index, column = 1, sticky =gstic)
        ttk.Label(panel,text="cs").grid(row = row_index, column = 2, sticky =gstic)

        return panel


    # -------------------------------------------------------------------------------
    # make a new frame and put buttons into it, but stay in this class
    def _newReadWriteButtonsPanel(self, container):
        cframe = ttk.Frame(container, borderwidth=3, relief='ridge')

        ttk.Button(cframe,text="Scan Spectrum" , command=self._clicScanSpectrum ).pack(side='left')
        ttk.Button(cframe,text="Help" , command=self._clicHelp).pack(side='left')

        return cframe
    
    # --------------------------------------------------------------------------------------
    def _clicScanSpectrum(self):
        cmd = Qsk5_req_spectrum_scan(self._edit_start_frequency.get(), self._edit_step_F_1x.get(), self._edit_samples_count.get(), self._edit_stime_cs.get())
        self.stat.qpypoll.queuePut(cmd)

    # --------------------------------------------------------------------------------------
    def _clicHelp(self):
        self.cmd_println("Available commands")

    # -------------------------------------------------------------------
    # implement the config savable
    def appImportConfig(self, cfg : AppConfig ):
        adict = cfg.getDictValue("spectrum_cfg", {})
        
        try:
            self._toplevel.setGeometry(adict['gui_cfg'])
        except Exception as _exc :
            self._spectrum_println("spectrum.appImportConfig"+str(_exc))            

    # -------------------------------------------------------------------
    # implement the config savable
    def appSaveConfig(self, cfg : AppConfig ):
        adict = dict()

        adict['gui_cfg'] = self._toplevel.getGeometry()

        cfg.setDictValue("spectrum_cfg", adict )        
        
        
        
            
    
    

        
        

